@@ -168,19 +168,18 @@ static int do_fcntl(const args_t *args, const int fd)
 		ret = fcntl(fd, F_SETOWN_EX, &owner);
 		check_return(args, ret, "F_SETOWN_EX, F_OWNER_PID");
 #endif
-
 #if defined(HAVE_GETPGRP) && defined(F_OWNER_PGRP)
 		owner.type = F_OWNER_PGRP;
 		owner.pid = getpgrp();
 		ret = fcntl(fd, F_SETOWN_EX, &owner);
 		check_return(args, ret, "F_SETOWN_EX, F_OWNER_PGRP");
-#elif defined(HAVE_GETPGRP) && defined(F_OWNER_GID)
+#endif
+#if defined(HAVE_GETPGRP) && defined(F_OWNER_GID)
 		owner.type = F_OWNER_GID;
 		owner.pid = getpgrp();
 		ret = fcntl(fd, F_SETOWN_EX, &owner);
 		check_return(args, ret, "F_SETOWN_EX, F_OWNER_GID");
 #endif
-
 #if defined(F_OWNER_TID) && defined(__linux__)
 		owner.type = F_OWNER_TID;
 		owner.pid = shim_gettid();
@@ -203,13 +202,13 @@ static int do_fcntl(const args_t *args, const int fd)
 		owner.type = F_OWNER_PGRP;
 		ret = fcntl(fd, F_GETOWN_EX, &owner);
 		check_return(args, ret, "F_GETOWN_EX, F_OWNER_PGRP");
-#elif defined(F_OWNER_GID)
+#endif
+#if defined(F_OWNER_GID)
 		owner.type = F_OWNER_GID;
 		ret = fcntl(fd, F_GETOWN_EX, &owner);
 		check_return(args, ret, "F_GETOWN_EX, F_OWNER_GID");
 #endif
-
-#if defined(__linux__)
+#if defined(F_OWNER_TID) && defined(__linux__)
 		owner.type = F_OWNER_TID;
 		ret = fcntl(fd, F_GETOWN_EX, &owner);
 		check_return(args, ret, "F_GETOWN_EX, F_OWNER_TID");
